//******************************************************************************
//Name          : Correlation.cs
//Description   : Implementation to experience the beauty and the pitfalls of Correlation
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

using Display;


class Correlation
{
    const int samples = 1024;

    MyRandom.Random noise;

    // Signale
    public double[] x;
    public double[] y;
    public double[] arg;

    // Korrelationsverlufe
    public double[] corrxy;
    public double[] corrxx;
    public double[] corryy;

    // Korrelationskoeffizienten
    public double rxy;
    public double rxx;
    public double ryy;

    // nur fr Anzeige
    public double xmin;
    public double xmax;

    // c'tor
    public Correlation()
    {
        noise = new MyRandom.Random(0.71036, 0.23906);

        x = new double[samples];
        y = new double[samples];
        arg = new double[samples];
        corrxy = new double[samples];
        corrxx = new double[samples];
        corryy = new double[samples];

        rxy = 0;
        rxx = 0;
        ryy = 0;

        xmin = 1.0E20;
        xmax = -1.0E20;
    }

    public void CreateSignal(double[] x, int select, bool checkminmax)
    {
        const double PI = 3.141592653589793;

        // Gerade
        if (select == 1)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = 1.7*k + 10;
            }
        }

        // Gerade + Rauschen
        if (select == 2)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = 1.7*k + 10 + 10*noise.Normal(0, 3);
            }
        }

        // negative Gerade
        if (select == 3)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = -1.7*k + 20;
            }
        }

        // negative Gerade + Rauschen
        if (select == 4)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = -1.7*k + 20 + 10*noise.Normal(0, 3);
            }
        }

        // sinusfrmig
        if (select == 5)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = Math.Sin(5*PI*k/samples);
            }
        }

        // sinusfrmig + Rauschen
        if (select == 6)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = Math.Sin(8*PI*k/samples) + 0.2*noise.Normal(0, 1);
            }
        }

        // cosinusfrmig
        if (select == 7)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = Math.Cos(5*PI*k/samples);
            }
        }

        // cosinusfrmig + Rauschen
        if (select == 8)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = Math.Cos(8*PI*k/samples) + 0.2*noise.Normal(0, 1);
            }
        }

        // 1/8-Puls
        if (select == 9)
        {
            for (int k=0; k<samples; k++)
            {
                if (k > samples/2 && k <= samples/2 + samples/8)
                {
                    x[k] = 1;
                }
            }
        }

        // 1/8-Puls + Rauschen
        if (select == 10)
        {
            for (int k=0; k<samples; k++)
            {
                if (k > samples/2 && k <= samples/2 + samples/8)
                {
        	        x[k] = 1;
                }

                x[k] += 0.4*noise.Normal(0, 1);
            }
        }

        // Rauschen
        if (select == 11)
        {
            for (int k=0; k<samples; k++)
            {
                x[k] = noise.Normal(0, 1);
            }
        }

        // determine xmin and xmax for displaying the data in scatterplot
        if (checkminmax)
        {
            for (int k=0; k<samples; k++)
            {
                if (x[k] > xmax) xmax = x[k];
                if (x[k] < xmin) xmin = x[k];
            }
        }

        // store argument data
        for (int k = 0; k < samples; k++)
        {
            arg[k] = k;
        }
    }

    public void CalculateZTransformation(double[] w)
    {
        double m = 0;	// Mittelwert
        double s = 0;	// Streuung

        // Berechnung Mittelwert
        for (int k=0; k<samples; k++)
        {
            m += w[k];
        }
        m /= samples;

        // Berechnung Streuung
        for (int k=0; k<samples; k++)
        {
            s += (w[k]-m)*(w[k]-m);
        }
        s /= (samples-1);
        s = Math.Sqrt(s);

        // z-Transformation
        for (int k=0; k<samples; k++)
        {
            w[k] = (w[k]-m)/s;
        }
    }

    public void CalculateCorrelation()
    {
        double sumx  = 0;
        double sumy  = 0;
        double sumxx = 0;
        double sumyy = 0;
        double sumxy = 0;

        for (int k=0; k<samples; k++)
        {
            sumx  += x[k];
            sumy  += y[k];
            sumxx += x[k]*x[k];
            sumyy += y[k]*y[k];
            sumxy += x[k]*y[k];
        }

        // Korrelation nach Pearson
        rxy = (samples*sumxy - sumx*sumy)
              /
              (Math.Sqrt((samples*sumxx - sumx*sumx) * (samples*sumyy - sumy*sumy)));

        // Korrelation als Skalarprodukt in Abhngigkeit
        // von der zeitlichen Verschiebung m
        for (int m=0; m<samples; m++)
        {
            for (int k=m; k<samples; k++)
            {
                corrxy[m] += x[k]*y[k-m];
                corrxx[m] += x[k]*x[k-m];
                corryy[m] += y[k]*y[k-m];
            }

            corrxy[m] /= samples;
            corrxx[m] /= samples;
            corryy[m] /= samples;
        }
    }
}


class Execute
{
    [STAThread]
    static void Main()
    {
        string userinput = "";
        int select = -1;

        while (select <= 11)
        {
            Correlation myCorrelation = new Correlation();

            Console.Write("Korrelationsanalyse\n\n");
            Console.Write("Bitte 1. Signal auswhlen:\n\n");
            Console.Write("Gerade                 ->  1\n");
            Console.Write("Gerade + Rauschen      ->  2\n");
            Console.Write("neg. Gerade            ->  3\n");
            Console.Write("neg. Gerade + Rauschen ->  4\n");
            Console.Write("Sinus                  ->  5\n");
            Console.Write("Sinus + Rauschen       ->  6\n");
            Console.Write("Cosinus                ->  7\n");
            Console.Write("Cosinus + Rauschen     ->  8\n");
            Console.Write("Puls                   ->  9\n");
            Console.Write("Puls + Rauschen        -> 10\n");
            Console.Write("Rauschen               -> 11\n\n");
            userinput = Console.ReadLine();
            select = Convert.ToInt32(userinput);

            myCorrelation.CreateSignal(myCorrelation.x, select, true);

            DisplayChart mydisplay1 = new DisplayChart(720, 480, SeriesChartType.Line, "Time domain", "k", "x(k)", 0, myCorrelation.x.Length);
            mydisplay1.AddData(myCorrelation.arg, myCorrelation.x);
            Application.Run(mydisplay1);
            mydisplay1.Dispose();

            for (int i = 0; i < 20; i++) Console.WriteLine();

            Console.WriteLine();
            Console.Write("Bitte 2. Signal auswhlen:\n\n");
            Console.Write("Gerade                 ->  1\n");
            Console.Write("Gerade + Rauschen      ->  2\n");
            Console.Write("neg. Gerade            ->  3\n");
            Console.Write("neg. Gerade + Rauschen ->  4\n");
            Console.Write("Sinus                  ->  5\n");
            Console.Write("Sinus + Rauschen       ->  6\n");
            Console.Write("Cosinus                ->  7\n");
            Console.Write("Cosinus + Rauschen     ->  8\n");
            Console.Write("Puls                   ->  9\n");
            Console.Write("Puls + Rauschen        -> 10\n");
            Console.Write("Rauschen               -> 11\n\n");
            Console.Write("no 2nd signal          ->  0\n\n");
            userinput = Console.ReadLine();
            select = Convert.ToInt32(userinput);

            if (select != 0)
            {
                myCorrelation.CreateSignal(myCorrelation.y, select, false);

                DisplayChart mydisplay2 = new DisplayChart(720, 480, SeriesChartType.Line, "Time domain", "k", "y(k)", 0, myCorrelation.y.Length);
                mydisplay2.AddData(myCorrelation.arg, myCorrelation.y);
                Application.Run(mydisplay2);
                mydisplay2.Dispose();

                DisplayChart mydisplay3 = new DisplayChart(720, 640, SeriesChartType.Point, "Scatterplot", "x(k)", "y(k)", myCorrelation.xmin, myCorrelation.xmax);
                mydisplay3.AddData(myCorrelation.x, myCorrelation.y);
                Application.Run(mydisplay3);
                mydisplay3.Dispose();
            }

            myCorrelation.CalculateZTransformation(myCorrelation.x);
            myCorrelation.CalculateZTransformation(myCorrelation.y);
            myCorrelation.CalculateCorrelation();

            DisplayChart mydisplay4 = new DisplayChart(720, 480, SeriesChartType.Line, "Time domain", "m", "corrxx(m)", 0, myCorrelation.y.Length);
            mydisplay4.AddData(myCorrelation.arg, myCorrelation.corrxx);
            Application.Run(mydisplay4);
            mydisplay4.Dispose();

            Application.Exit();
            for (int i = 0; i < 20; i++) Console.WriteLine();
        }

        Console.WriteLine();
        Console.WriteLine("Ende...<ENTER>");
        Console.ReadLine();
    }
}
